@@ -5,6 +5,7 @@ import avi.migrationtools.f5_converter.converter_constants as conv_const
 from pkg_resources import parse_version
 from avi.migrationtools.f5_converter.profile_converter import ssl_count
 from avi.migrationtools.f5_converter.conversion_util import F5Util
+from avi.migrationtools.avi_migration_utils import update_count
 
 LOG = logging.getLogger(__name__)
 # Creating f5 object for util library.
@@ -63,6 +64,19 @@ class MonitorConfigConv(object):
                          name):
         pass
 
+    def update_request_for_avi(self, send, is_http):
+        if not send:
+            return None
+        send = send.replace('\\\\', '\\')
+        send = send.replace('"', '')
+        if send == 'none':
+            send = None
+        if is_http and send and 'HTTP' not in send:
+            send = send.replace('\\r\\n', ' HTTP/1.1\\r\\n', 1)
+        if is_http and send and 'BigIP' in send:
+            send = send.replace("BigIP","avi1.0")
+        return send
+
     def create_sslprofile(self, monitor_dict, f5_monitor, avi_config,
                            tenant, cloud_name, merge_object_mapping, sys_dict):
         """
@@ -176,7 +190,8 @@ class MonitorConfigConv(object):
         LOG.info('Added new SSL key and certificate for %s' % name)
 
     def convert(self, f5_config, avi_config, input_dir, user_ignore, tenant,
-                cloud_name, controller_version, merge_object_mapping, sys_dict):
+                cloud_name, controller_version, merge_object_mapping, sys_dict,
+                custom_mappings=None):
         """
 
         :param f5_config:  parsed f5 config dict
@@ -188,6 +203,7 @@ class MonitorConfigConv(object):
         :param controller_version: controller version.
         :param merge_object_mapping: flag for merge object
         :param sys_dict: baseline profile.
+        :param custom_mappings: custom mappings to overwrite monitor config.
         :return:
         """
         LOG.debug("Converting health monitors")
@@ -198,6 +214,9 @@ class MonitorConfigConv(object):
         # Added varibles to  get total count of object.
         total_size = len(monitor_config.keys())
         progressbar_count = 0
+        custom_config = custom_mappings.get(
+            conv_const.HM_CUSTOM_KEY, dict()
+        ) if custom_mappings else dict()
         for key in monitor_config.keys():
             progressbar_count += 1
             # Added call to check progress.
@@ -218,6 +237,33 @@ class MonitorConfigConv(object):
                 continue
             f5_monitor = self.get_defaults(monitor_config, key)
             monitor_type, name = self.get_name_type(f5_monitor, key)
+            if '/' in monitor_type:
+                monitor_type = monitor_type.split('/')[-1]
+            m_tenant, m_name = conv_utils.get_tenant_ref(name)
+            # Check if custom cofig present for this HM
+            r_hm = [obj for obj in custom_config if
+                            obj['monitor_name'] == m_name]
+            if r_hm:
+                LOG.debug(
+                    "Found custom config for %s replacing with custom config"
+                    % m_name)
+                r_hm = r_hm[0]
+                avi_monitor = r_hm['avi_config']
+                # Added prefix for objects
+                if self.prefix:
+                    avi_monitor['name'] = self.prefix + '-' + m_name
+                else:
+                    avi_monitor['name'] = m_name
+                if tenant:
+                    m_tenant = tenant
+                avi_monitor['tenant_ref'] = conv_utils.get_object_ref(
+                    m_tenant, 'tenant')
+                avi_config["HealthMonitor"].append(avi_monitor)
+                conv_utils.add_conv_status(
+                    'monitor', monitor_type, m_name, {
+                        'status': conv_const.STATUS_SUCCESSFUL
+                    }, [{'health_monitor': avi_monitor}])
+                continue
             # Added prefix for objects
             if self.prefix:
                 name = self.prefix + '-' + name
@@ -265,6 +311,7 @@ class MonitorConfigConv(object):
                     avi_config["HealthMonitor"].append(avi_monitor)
                 LOG.debug("Conversion successful for monitor: %s" % name)
             except:
+                update_count('error')
                 LOG.error("Failed to convert monitor: %s" % key, exc_info=True)
                 if name:
                     conv_utils.add_status_row('monitor', monitor_type, name,
@@ -293,6 +340,8 @@ class MonitorConfigConv(object):
         :return: monitor_dict
         """
         monitor_type, name = self.get_name_type(f5_monitor, key)
+        if '/' in monitor_type:
+            monitor_type = monitor_type.split('/')[-1]
         skipped = [val for val in f5_monitor.keys()
                    if val not in self.supported_attributes]
         indirect = copy.deepcopy(self.indirect_mappings)
@@ -318,7 +367,7 @@ class MonitorConfigConv(object):
         # Added prefix for objects
         if self.prefix:
             name = self.prefix + '-' + name
-        if tenant_ref != 'admin':
+        if tenant_ref:
             tenant = tenant_ref
         monitor_dict['tenant_ref'] = conv_utils.get_object_ref(tenant, 'tenant')
         monitor_dict["name"] = name
@@ -354,7 +403,7 @@ class MonitorConfigConv(object):
             na_list = self.na_https
             u_ignore = user_ignore.get("https", [])
             skipped = self.convert_https(monitor_dict, f5_monitor, skipped,
-                      avi_config, tenant_ref, input_dir, cloud_name,
+                      avi_config, tenant, input_dir, cloud_name,
                       controller_version, merge_object_mapping, sys_dict)
         elif monitor_type == "dns":
             na_list = self.na_dns
@@ -481,11 +530,10 @@ class MonitorConfigConvV11(MonitorConfigConv):
         """
         skipped = [key for key in skipped if key not in self.http_attr]
         send = f5_monitor.get('send', 'HEAD / HTTP/1.0')
-        send = send.replace('\\\\', '\\')
-        send = send.replace('"', '')
-        send = conv_utils.rreplace(send, '\\r\\n', '', 1)
+        send = self.update_request_for_avi(send, True)
         monitor_dict["type"] = "HEALTH_MONITOR_HTTP"
         monitor_dict["http_monitor"] = {
+            "exact_http_request": True,
             "http_request": send,
             "http_response_code": ["HTTP_2XX", "HTTP_3XX"]}
         destination = f5_monitor.get(self.dest_key, "*:*")
@@ -526,11 +574,10 @@ class MonitorConfigConvV11(MonitorConfigConv):
         """
         skipped = [key for key in skipped if key not in self.https_attr]
         send = f5_monitor.get('send', 'HEAD / HTTP/1.0')
-        send = send.replace('\\\\', '\\')
-        send = send.replace('"', '')
-        send = conv_utils.rreplace(send, '\\r\\n', '', 1)
+        send = self.update_request_for_avi(send, True)
         monitor_dict["type"] = "HEALTH_MONITOR_HTTPS"
         monitor_dict["https_monitor"] = {
+            "exact_http_request": True,
             "http_request": send,
             "http_response_code": ["HTTP_2XX", "HTTP_3XX"]}
         monitor_dict["https_monitor"]['ssl_attributes'] = dict()
@@ -625,10 +672,10 @@ class MonitorConfigConvV11(MonitorConfigConv):
             monitor_dict["monitor_port"] = dest_str[1]
         monitor_dict["type"] = "HEALTH_MONITOR_TCP"
         request = f5_monitor.get("send", None)
-        if request:
-            request = request.replace('\\\\', '\\')
-            request = conv_utils.rreplace(request, '\\r\\n', '', 1)
+        request = self.update_request_for_avi(request, False)
         response = f5_monitor.get("recv", None)
+        if response == 'none':
+            response = None
         tcp_monitor = None
         if request or response:
             tcp_monitor = {"tcp_request": request, "tcp_response": response}
@@ -638,9 +685,9 @@ class MonitorConfigConvV11(MonitorConfigConv):
         if tcp_monitor:
             if maintenance_resp:
                 tcp_monitor["maintenance_response"] = maintenance_resp
-            tcp_monitor["http_response"] = http_rsp
+            tcp_monitor["tcp_response"] = http_rsp
         else:
-            tcp_monitor = {"http_response": http_rsp}
+            tcp_monitor = {"tcp_response": http_rsp}
             if maintenance_resp:
                 tcp_monitor["maintenance_response"] = maintenance_resp
             monitor_dict["tcp_monitor"] = tcp_monitor
@@ -676,9 +723,10 @@ class MonitorConfigConvV11(MonitorConfigConv):
             monitor_dict["monitor_port"] = dest_str[1]
         monitor_dict["type"] = "HEALTH_MONITOR_UDP"
         request = f5_monitor.get("send", None)
-        request = request.replace('\\\\', '\\')
-        request = conv_utils.rreplace(request, '\\r\\n', '', 1)
+        request = self.update_request_for_avi(request, False)
         response = f5_monitor.get("recv", None)
+        if response == 'none':
+            response = None
         udp_monitor = None
         if request or response:
             udp_monitor = {"udp_request": request, "udp_response": response}
@@ -688,9 +736,9 @@ class MonitorConfigConvV11(MonitorConfigConv):
         if udp_monitor:
             if maintenance_resp:
                 udp_monitor["maintenance_response"] = maintenance_resp
-            udp_monitor["http_response"] = http_rsp
+            udp_monitor["udp_response"] = http_rsp
         else:
-            udp_monitor = {"http_response": http_rsp}
+            udp_monitor = {"udp_response": http_rsp}
             if maintenance_resp:
                 udp_monitor["maintenance_response"] = maintenance_resp
             monitor_dict["udp_monitor"] = udp_monitor
@@ -840,6 +888,8 @@ class MonitorConfigConvV10(MonitorConfigConv):
                 parent_monitor = copy.deepcopy(parent_monitor)
                 parent_monitor.update(f5_monitor)
                 f5_monitor = parent_monitor
+            else:
+                f5_monitor["type"] = parent_name
         else:
             f5_monitor["type"] = key
         return f5_monitor
@@ -863,11 +913,10 @@ class MonitorConfigConvV10(MonitorConfigConv):
         http_attr = self.http_attr + ignore_list
         skipped = [key for key in skipped if key not in http_attr]
         send = f5_monitor.get('send', 'HEAD / HTTP/1.0')
-        send = send.replace('\\\\', '\\')
-        send = send.replace('"', '')
-        send = conv_utils.rreplace(send, '\\r\\n', '', 1)
+        send = self.update_request_for_avi(send, False)
         monitor_dict["type"] = "HEALTH_MONITOR_HTTP"
         monitor_dict["http_monitor"] = {
+            "exact_http_request": True,
             "http_request": send,
             "http_response_code": ["HTTP_2XX", "HTTP_3XX"]
         }
@@ -911,11 +960,10 @@ class MonitorConfigConvV10(MonitorConfigConv):
         https_attr = ignore_list + self.https_attr
         skipped = [key for key in skipped if key not in https_attr]
         send = f5_monitor.get('send', None)
-        send = send.replace('\\\\', '\\')
-        send = send.replace('"', '')
-        send = conv_utils.rreplace(send, '\\r\\n', '', 1)
+        send = self.update_request_for_avi(send, False)
         monitor_dict["type"] = "HEALTH_MONITOR_HTTPS"
         monitor_dict["https_monitor"] = {
+            "exact_http_request": True,
             "http_request": send,
             "http_response_code": ["HTTP_2XX", "HTTP_3XX"]
         }
@@ -975,14 +1023,14 @@ class MonitorConfigConvV10(MonitorConfigConv):
             monitor_dict["monitor_port"] = dest_str[1]
         monitor_dict["type"] = "HEALTH_MONITOR_TCP"
         request = f5_monitor.get("send", None)
-        if request:
-            request = request.replace('\\\\', '\\')
-            request = conv_utils.rreplace(request, '\\r\\n', '', 1)
+        request = self.update_request_for_avi(request, False)
         response = f5_monitor.get("recv", None)
         tcp_monitor = None
         if request or response:
             request = request.replace('\"', '') if request else None
             response = response.replace('\"', '') if response else None
+            if response == 'none':
+                response = None
             tcp_monitor = {"tcp_request": request, "tcp_response": response}
             monitor_dict["tcp_monitor"] = tcp_monitor
         # Added mapping for http_response.
@@ -1027,13 +1075,14 @@ class MonitorConfigConvV10(MonitorConfigConv):
             monitor_dict["monitor_port"] = dest_str[1]
         monitor_dict["type"] = "HEALTH_MONITOR_UDP"
         request = f5_monitor.get("send", None)
-        request = request.replace('\\\\', '\\')
-        request = conv_utils.rreplace(request, '\\r\\n', '', 1)
+        request = self.update_request_for_avi(request, False)
         response = f5_monitor.get("recv", None)
         udp_monitor = None
         if request or response:
             request = request.replace('\"', '') if request else None
             response = response.replace('\"', '') if response else None
+            if response == 'none':
+                response = None
             udp_monitor = {"udp_request": request, "udp_response": response}
             monitor_dict["udp_monitor"] = udp_monitor
         # Added mapping for http_response.
