@@ -3,6 +3,7 @@ import avi.migrationtools.f5_converter.converter_constants as final
 import avi.migrationtools.f5_converter.converter_constants as conv_const
 from avi.migrationtools.f5_converter.profile_converter import ProfileConfigConv
 from avi.migrationtools.f5_converter.conversion_util import F5Util
+from avi.migrationtools.avi_migration_utils import update_count
 LOG = logging.getLogger(__name__)
 # Creating f5 object for util library.
 conv_utils = F5Util()
@@ -78,6 +79,12 @@ class PersistenceConfigConv(object):
             msg = "persistence conversion started..."
             conv_utils.print_progress_bar(progressbar_count, total_size, msg,
                              prefix='Progress', suffix='')
+            if key == 'global-settings':
+                msg = 'persist mode not supported skipping conversion: %s' \
+                      % name
+                LOG.warning(msg)
+                self.update_conv_status_for_skip(None, name, msg)
+                continue
             try:
                 persist_mode, name = key.split(" ")
                 LOG.debug("Converting persistence profile: %s" % name)
@@ -86,7 +93,7 @@ class PersistenceConfigConv(object):
                 profile = prof_conv.update_with_default_profile(
                     persist_mode, profile, f5_persistence_dict, name)
                 tenant, name = conv_utils.get_tenant_ref(name)
-                if tenant_ref != 'admin':
+                if tenant_ref:
                     tenant = tenant_ref
                 if self.prefix:
                     name = '{}-{}'.format(self.prefix, name)
@@ -146,6 +153,7 @@ class PersistenceConfigConv(object):
                 self.update_conversion_status(conv_status, persist_mode,
                                               name, persist_profile)
             except:
+                update_count('error')
                 LOG.error("Failed to convert persistance profile : %s" % key,
                           exc_info=True)
                 self.update_conv_status_for_error(name, persist_mode, key)
@@ -227,8 +235,8 @@ class PersistenceConfigConvV11(PersistenceConfigConv):
         timeout = parent_obj.convert_timeout(timeout)
         persist_profile = {
             "name": name,
-            "app_cookie_persistence_profile": {
-                "prst_hdr_name": cookie_name,
+            "http_cookie_persistence_profile": {
+                "cookie_name": cookie_name,
                 "timeout": timeout
             },
             "server_hm_down_recovery": "HM_DOWN_PICK_NEW_SERVER",
@@ -277,6 +285,7 @@ class PersistenceConfigConvV11(PersistenceConfigConv):
         skipped += [attr for attr in profile.keys()
                     if attr not in supported_attr]
         timeout = profile.get("timeout", final.SOURCE_ADDR_TIMEOUT)
+        timeout = 0 if timeout == 'indefinite' else timeout
         if timeout > 0:
             timeout = int(timeout)/final.SEC_IN_MIN
         persist_profile = {
@@ -377,8 +386,8 @@ class PersistenceConfigConvV10(PersistenceConfigConv):
         timeout = parent_obj.convert_timeout(timeout)
         persist_profile = {
             "name": name,
-            "app_cookie_persistence_profile": {
-                "prst_hdr_name": cookie_name,
+            "http_cookie_persistence_profile": {
+                "cookie_name": cookie_name,
                 "timeout": timeout
             },
             "server_hm_down_recovery": "HM_DOWN_PICK_NEW_SERVER",
