@@ -3,6 +3,7 @@ import copy
 import re
 import avi.migrationtools.f5_converter.converter_constants as conv_const
 from avi.migrationtools.f5_converter.conversion_util import F5Util
+from avi.migrationtools.avi_migration_utils import update_count
 LOG = logging.getLogger(__name__)
 # Creating f5 object for util library.
 conv_utils = F5Util()
@@ -87,6 +88,7 @@ class PoolConfigConv(object):
                     avi_config['PoolGroup'].extend(converted_objs['pg_obj'])
                 LOG.debug("Conversion successful for Pool: %s" % pool_name)
             except:
+                update_count('error')
                 LOG.error("Failed to convert pool: %s" % pool_name,
                           exc_info=True)
                 conv_utils.add_status_row('pool', None, pool_name,
@@ -133,9 +135,8 @@ class PoolConfigConv(object):
             if monitor_obj:
                 tenant = conv_utils.get_name(
                     monitor_obj[0]['tenant_ref'])
-                monitor_refs.append(conv_utils.get_object_ref(
-                    monitor_obj[0]['name'], 'healthmonitor',
-                    tenant=tenant))
+                monitor_refs.append(conv_utils.get_object_ref(monitor_obj[0]['name'],
+                                                              'healthmonitor', tenant=tenant))
             else:
                 LOG.warning("Monitor not found: %s for pool %s" %
                             (monitor, pool_name))
@@ -167,7 +168,10 @@ class PoolConfigConv(object):
             'lb_algorithm': algo,
             'cloud_ref': conv_utils.get_object_ref(cloud_ref, 'cloud')
         }
-        if not tenant_ref == 'admin':
+
+        if any(server["port"] == "" for server in servers):
+            pool_obj.update({"use_service_port": "true"})
+        if tenant_ref:
             tenant = tenant_ref
         pool_obj['tenant_ref'] = conv_utils.get_object_ref(tenant, 'tenant')
         if ramp_time:
@@ -177,7 +181,15 @@ class PoolConfigConv(object):
                 limits['connection_limit']
         if limits.get('rate_limit', 0) > 0:
             pool_obj['max_conn_rate_per_server'] = {
-                'count': limits['rate_limit']
+                'count': limits['rate_limit'],
+                'explicit_tracking': False,
+                'period': 1,
+                'action': {
+                    'status_code': 'HTTP_LOCAL_RESPONSE_STATUS_CODE_429',
+                    'type': "RL_ACTION_NONE"
+                },
+                'burst_sz': 0,
+                'fine_grain': False
             }
         return pool_obj
 
@@ -354,7 +366,7 @@ class PoolConfigConvV11(PoolConfigConv):
             status_flag = True
         tenant, name = conv_utils.get_tenant_ref(pool_name)
         tenant_name = tenant
-        if not tenant_ref == 'admin':
+        if tenant_ref:
             tenant = tenant_ref
         num_retries = f5_pool.get('reselect-tries', None)
         if num_retries:
@@ -385,12 +397,15 @@ class PoolConfigConvV11(PoolConfigConv):
                   else None
         if vrf:
             vrf_ref = conv_utils.get_object_ref(vrf, 'vrfcontext',
-                                                tenant_name=tenant_name,
+                                                tenant=tenant_name,
                                                 cloud_name=cloud_ref)
         else:
             vrf_ref = conv_utils.get_vrf_context_ref(address, vrf_config, 'pool',
                                                  pool_name, cloud_ref)
-        
+        if not vrf_ref:
+            vrf_ref = conv_utils.get_object_ref("global", 'vrfcontext',
+                                                tenant=tenant_name,
+                                                cloud_name=cloud_ref)
         if vrf_ref:
             pool_obj["vrf_ref"] = vrf_ref
 
@@ -458,10 +473,12 @@ class PoolConfigConvV11(PoolConfigConv):
         connection_limit = []
         server_skipped = []
         for server_name in servers_config.keys():
+            hostname = None
             server = servers_config[server_name]
             parts = server_name.split(':')
             node = nodes.get(parts[0], None)
-            if node:
+            if node and node.get("address"):
+                hostname = parts[0].split('/')[-1]
                 if '%' in node["address"]:
                     ip_addr, vrf = node["address"].split('%')
                     conv_utils.add_vrf(avi_config, vrf, cloud_ref)
@@ -484,6 +501,8 @@ class PoolConfigConvV11(PoolConfigConv):
                             if orig_port else " no port"))
                 server_skipped.append(server_name)
                 continue
+            if port == '0':
+                port = ""
             enabled = True
             state = server.get("state", 'enabled')
             session = server.get("session", 'enabled')
@@ -498,6 +517,7 @@ class PoolConfigConvV11(PoolConfigConv):
                 LOG.warning('Avi does not support IPv6. Replace 1.1.1.1 '
                             'ipv4 for : %s' % ip_addr)
                 ip_addr = '1.1.1.1'
+
             server_obj = {
                 'ip': {
                     'addr': ip_addr,
@@ -508,6 +528,9 @@ class PoolConfigConvV11(PoolConfigConv):
                 'port': port
             }
 
+            if hostname:
+                server_obj['hostname'] = hostname
+
             if priority:
                 server_obj['priority'] = priority
             ratio = server.get("ratio", None)
