@@ -4,6 +4,8 @@ import os
 import yaml
 import avi.migrationtools.f5_converter.converter_constants as final
 from avi.migrationtools.f5_converter.conversion_util import F5Util
+from avi.migrationtools.avi_migration_utils import update_count
+
 LOG = logging.getLogger(__name__)
 ssl_count = {'count': 0}
 # Creating f5 object for util library.
@@ -13,7 +15,8 @@ conv_utils = F5Util()
 class ProfileConfigConv(object):
     @classmethod
     def get_instance(cls, version, f5_profile_attributes,
-                     object_merge_check, prefix, keypassphrase):
+                     object_merge_check, prefix, keypassphrase, skip_pki=False,
+                     distinct_app_profile=False):
         """
 
         :param version:  version of f5 instance
@@ -21,17 +24,19 @@ class ProfileConfigConv(object):
         :param object_merge_check: Flag for object merge
         :param prefix: prefix for objects
         :param keypassphrase: path of keypassphrase
+        :param skip_pki: PKI profile migration needs to be skipped
+        :param distinct_app_profile: Merge duplicates needs to be slipped
         :return: object of respective f5 version object.
         """
         f5_profile_attributes = f5_profile_attributes
         if version == '10':
             return ProfileConfigConvV10(
                 f5_profile_attributes, object_merge_check, prefix,
-                keypassphrase)
+                keypassphrase, skip_pki, distinct_app_profile)
         if version in ['11', '12']:
             return ProfileConfigConvV11(
                 f5_profile_attributes, object_merge_check, prefix,
-                keypassphrase)
+                keypassphrase, skip_pki, distinct_app_profile)
 
     default_key = None
 
@@ -81,7 +86,7 @@ class ProfileConfigConv(object):
             try:
                 profile_type, name = key.split(" ")
                 tenant, name = conv_utils.get_tenant_ref(name)
-                if not tenant_ref == 'admin':
+                if tenant_ref:
                     tenant = tenant_ref
                 if profile_type not in self.supported_types:
                     msg = ("Skipped not supported profile: %s of type: %s"
@@ -96,6 +101,14 @@ class ProfileConfigConv(object):
                     name = self.prefix + '-' + name
                 LOG.debug("Converting profile: %s" % name)
                 profile = profile_config[key]
+                if not profile:
+                    LOG.warn('Empty config for profile %s Skipping the config'
+                             % name)
+                    conv_utils.add_status_row(
+                        'profile', profile_type, name,
+                        final.STATUS_NOT_APPLICABLE, 'Empty config')
+                    continue
+                # print key, profile
                 profile = self.update_with_default_profile(
                     profile_type, profile, profile_config, name)
                 u_ignore = user_ignore.get('profile', {})
@@ -105,6 +118,7 @@ class ProfileConfigConv(object):
                     merge_object_mapping, sys_dict)
                 LOG.debug("Conversion successful for profile: %s" % name)
             except:
+                update_count('error')
                 LOG.error("Failed to convert profile: %s" % key, exc_info=True)
                 if name:
                     conv_utils.add_status_row('profile', profile_type, name,
@@ -138,6 +152,8 @@ class ProfileConfigConv(object):
         :return: Complete profile with updated attributes from defaults
         """
         parent_name = profile.get(self.default_key, None)
+        if parent_name and '/' in parent_name:
+            parent_name = parent_name.split('/')[-1]
         if parent_name and profile_name != parent_name:
             parent_profile = profile_config.get(profile_type + " " +
                                                 parent_name, None)
@@ -219,6 +235,8 @@ class ProfileConfigConv(object):
             LOG.warning('Create self cerificate and key for : %s' % name)
 
         ssl_kc_obj = None
+        if tenant == None:
+            tenant = 'admin'
         if key and cert:
             cert = {"certificate": cert}
             ssl_kc_obj = {
@@ -254,16 +272,99 @@ class ProfileConfigConv(object):
                 converted_objs.append({'ssl_cert_key': ssl_kc_obj})
                 avi_config['SSLKeyAndCertificate'].append(ssl_kc_obj)
 
+    def update_ca_cert_obj(self, name, ca_cert_file_name, input_dir, tenant,
+                           avi_config, converted_objs, merge_object_mapping,
+                           sys_dict):
+        """
+        This method create the certs if certificate not present at location
+        it create dummy certificate.
+        :param name: name of certificate.
+        :param key_file_name: name of keyfile of cert
+        :param cert_file_name: name of cert file
+        :param input_dir: location of cert and key
+        :param tenant: tenant name
+        :param avi_config: converted avi config dict
+        :param converted_objs: list of converted object profile
+        :param default_profile_name: name of default profile name.
+        :param key_and_cert_mapping_list: list of key and cert
+        :param merge_object_mapping: merged object dict for merging objects
+        :param sys_dict: baseline objects
+        :return:
+        """
+
+        cert_name = [cert['name'] for cert in avi_config['SSLKeyAndCertificate']
+                     if cert['name'] == name and
+                     cert['type'] == 'SSL_CERTIFICATE_TYPE_CA']
+
+        if cert_name:
+            LOG.warning(
+                'SSL ca cert is already exist for %s is %s' % (
+                    ca_cert_file_name, cert_name[0]))
+            return
+        folder_path = input_dir + os.path.sep
+        ca_cert = None
+        # Removed / from cert_file_name to get name of file.
+        if ca_cert_file_name:
+            if '/' in ca_cert_file_name:
+                ca_cert_file_name = ca_cert_file_name.split('/')[-1]
+            if ':' in ca_cert_file_name:
+                ca_cert_file_name = ca_cert_file_name.split(':')[-1]
+            ca_cert = conv_utils.upload_file(folder_path + ca_cert_file_name)
+
+        if ca_cert:
+            if not conv_utils.check_certificate_expiry(
+                    input_dir, ca_cert_file_name):
+                ca_cert = None
+
+        if not ca_cert:
+            key, ca_cert = conv_utils.create_self_signed_cert()
+            name += '-dummy'
+            LOG.warning('Create self cerificate and key for : %s' % name)
+
+        ca_cert_obj = None
+        cert_name = name
+        if ca_cert_file_name and '.crt' in ca_cert_file_name:
+            if ':' in ca_cert_file_name:
+                ca_cert_file_name = ca_cert_file_name.split(':')[-1]
+            ca_cert_file_name = '%s.crt' % ca_cert_file_name.split('.crt')[0]
+        if not ca_cert_file_name:
+            ca_cert_file_name = name
+        if ca_cert:
+            cert = {"certificate": ca_cert}
+            ca_cert_obj = {
+                'name': ca_cert_file_name,
+                'tenant_ref': conv_utils.get_object_ref(tenant, 'tenant'),
+                'certificate': cert,
+                'type': 'SSL_CERTIFICATE_TYPE_CA'
+            }
+            LOG.info('Added new ca certificate for %s' % name)
+        if ca_cert_obj and self.object_merge_check:
+            if 'dummy' not in ca_cert_obj['name']:
+                conv_utils.update_skip_duplicates(
+                    ca_cert_obj, avi_config['SSLKeyAndCertificate'],
+                    'ssl_cert_key', converted_objs, name, None,
+                    merge_object_mapping, None, self.prefix,
+                    sys_dict['SSLKeyAndCertificate'])
+            else:
+                converted_objs.append({'ssl_cert_key': ca_cert_obj})
+                avi_config['SSLKeyAndCertificate'].append(ca_cert_obj)
+            self.certkey_count += 1
+        else:
+            converted_objs.append({'ssl_cert_key': ca_cert_obj})
+            avi_config['SSLKeyAndCertificate'].append(ca_cert_obj)
+
 
 class ProfileConfigConvV11(ProfileConfigConv):
     def __init__(self, f5_profile_attributes, object_merge_check, prefix,
-                 keypassphrase):
+                 keypassphrase, skip_pki, distinct_app_profile):
         """
 
         :param f5_profile_attributes: f5 profile attributes from yaml file.
         :param object_merge_check: flag for merging objects
         :param prefix: prefix for objects
         :param keypassphrase: keypassphrase yaml file location
+        :param skip_pki: PKI profile migration needs to be skipped
+        :param distinct_app_profile: Merge duplicates needs to be slipped
         """
         self.supported_types = \
             f5_profile_attributes['Profile_supported_types']
@@ -313,6 +414,8 @@ class ProfileConfigConvV11(ProfileConfigConv):
         self.certkey_count = 0
         # Added prefix for objects
         self.prefix = prefix
+        self.skip_pki = skip_pki
+        self.distinct_app_profile = distinct_app_profile
 
     def convert_profile(self, profile, key, f5_config, profile_config,
                         avi_config, input_dir, user_ignore, tenant_ref,
@@ -341,7 +444,7 @@ class ProfileConfigConvV11(ProfileConfigConv):
         parent_cls = super(ProfileConfigConvV11, self)
         profile_type, name = key.split(' ')
         tenant, name = conv_utils.get_tenant_ref(name)
-        if not tenant_ref == 'admin':
+        if tenant_ref:
             tenant = tenant_ref
         default_profile_name = '%s %s' % (profile_type,
                                           profile_type.replace('-', ''))
@@ -378,6 +481,11 @@ class ProfileConfigConvV11(ProfileConfigConv):
                 key_file = profile.get("key", None)
                 cert_file = None if cert_file == 'none' else cert_file
                 key_file = None if key_file == 'none' else key_file
+            cert_name = cert_file
+            if cert_name:
+                cert_name = cert_name.split('/')[-1]
+            else:
+                cert_name = name
             # Added for getting correct file names from cache path in sys file
             sys_file = f5_config.get('file', {})
             for file_key in sys_file:
@@ -392,9 +500,24 @@ class ProfileConfigConvV11(ProfileConfigConv):
                         cert_file = sys_file[file_key]['cache-path'].rsplit(
                                                                     '/', 1)[-1]
             parent_cls.update_key_cert_obj(
-                name, key_file, cert_file, input_dir, tenant_ref, avi_config,
-                converted_objs, default_profile_name, key_and_cert_mapping_list,
-                merge_object_mapping, sys_dict)
+                cert_name, key_file, cert_file, input_dir, tenant_ref,
+                avi_config, converted_objs, default_profile_name,
+                key_and_cert_mapping_list, merge_object_mapping, sys_dict)
+            if profile.get('chain', 'none') != 'none':
+                LOG.debug("Migrating root/intermediate cert for %s" % name)
+                ca_cert_file = None
+                for file_key in sys_file:
+                    file_type, file_name = file_key.split(' ')
+                    if (file_type == 'ssl-cert' and
+                            file_name == profile['chain'] and
+                            sys_file[file_key].get('cache-path')):
+                        ca_cert_file = sys_file[file_key][
+                            'cache-path'].rsplit('/', 1)[-1]
+                        break
+                self.update_ca_cert_obj(
+                    name, ca_cert_file, input_dir, tenant, avi_config,
+                    converted_objs, merge_object_mapping,
+                    sys_dict)
 
             # ciphers = profile.get('ciphers', 'DEFAULT')
             # ciphers = 'AES:3DES:RC4' if ciphers == 'DEFAULT' else ciphers
@@ -403,6 +526,8 @@ class ProfileConfigConvV11(ProfileConfigConv):
             ssl_profile['name'] = name
             ssl_profile['tenant_ref'] = conv_utils.get_object_ref(
                 tenant, 'tenant')
+            if cert_name:
+                ssl_profile['cert_name'] = cert_name
             ssl_profile['accepted_ciphers'] = self.ciphers
             close_notify = profile.get('unclean-shutdown', None)
             if close_notify and close_notify == 'enabled':
@@ -444,7 +569,7 @@ class ProfileConfigConvV11(ProfileConfigConv):
             else:
                 ca_file_name = None
 
-            if ca_file_name:
+            if ca_file_name and not self.skip_pki:
                 pki_profile = dict()
                 file_path = input_dir+os.path.sep+ca_file_name
                 pki_profile["name"] = name
@@ -499,8 +624,8 @@ class ProfileConfigConvV11(ProfileConfigConv):
             app_profile['description'] = profile.get('description', None)
             encpt_cookie = profile.get('encrypt-cookies', 'none')
             encpt_cookie = False if encpt_cookie == 'none' else True
-            xff_enabled = profile.get('accept-xff', 'disabled')
-            xff_enabled = False if xff_enabled == 'disabled' else True
+            # xff_enabled = profile.get('accept-xff', 'disabled')
+            # xff_enabled = False if xff_enabled == 'disabled' else True
             con_mltplxng = profile.get('oneconnect-transformations',
                                        'disabled')
             con_mltplxng = False if con_mltplxng == 'disabled' else True
@@ -508,10 +633,11 @@ class ProfileConfigConvV11(ProfileConfigConv):
             insert_xff = profile.get('insert-xforwarded-for', 'disabled')
             insert_xff = True if insert_xff == 'enabled' else False
             http_profile['x_forwarded_proto_enabled'] = insert_xff
-            http_profile['xff_alternate_name'] = \
-                profile.get('xff-alternative-names', None)
+            xff_alt_names = profile.get('xff-alternative-names', None)
+            xff_alt_names = None if xff_alt_names == 'none' else xff_alt_names
+            http_profile['xff_alternate_name'] = xff_alt_names
             http_profile['secure_cookie_enabled'] = encpt_cookie
-            http_profile['xff_enabled'] = xff_enabled
+            http_profile['xff_enabled'] = insert_xff
             http_profile['connection_multiplexing_enabled'] = con_mltplxng
             enforcement = profile.get('enforcement', None)
             if enforcement:
@@ -527,11 +653,11 @@ class ProfileConfigConvV11(ProfileConfigConv):
             header_erase = profile.get('header-erase', None)
             header_erase = None if header_erase == 'none' else header_erase
             if header_erase:
-                header_erase = header_erase.replace('\"', '')
+                header_erase = header_erase.replace('\"', '').strip()
             header_insert = profile.get('header-insert', None)
             header_insert = None if header_insert == 'none' else header_insert
             if header_insert:
-                header_insert = header_insert.replace('\"', '')
+                header_insert = header_insert.replace('\"', '').strip()
 
             if header_erase or header_insert:
                 rules = []
@@ -581,7 +707,7 @@ class ProfileConfigConvV11(ProfileConfigConv):
                 if host:
                     app_profile['fallback_host'] = host
             # code to merge application profile count.
-            if self.object_merge_check:
+            if self.object_merge_check and not self.distinct_app_profile:
                 conv_utils.update_skip_duplicates(
                     app_profile, avi_config['ApplicationProfile'],
                     'app_profile', converted_objs, name, default_profile_name,
@@ -606,7 +732,7 @@ class ProfileConfigConvV11(ProfileConfigConv):
             app_profile['type'] = 'APPLICATION_PROFILE_TYPE_DNS'
             app_profile['description'] = profile.get('description', None)
             # code to merge application profile count.
-            if self.object_merge_check:
+            if self.object_merge_check and not self.distinct_app_profile:
                 conv_utils.update_skip_duplicates(
                     app_profile, avi_config['ApplicationProfile'],
                     'app_profile', converted_objs, name, default_profile_name,
@@ -661,7 +787,7 @@ class ProfileConfigConvV11(ProfileConfigConv):
             http_profile["cache_config"] = cache_config
             app_profile["http_profile"] = http_profile
             # code to merge application profile count.
-            if self.object_merge_check:
+            if self.object_merge_check and not self.distinct_app_profile:
                 conv_utils.update_skip_duplicates(
                     app_profile, avi_config['ApplicationProfile'],
                     'app_profile', converted_objs, name, default_profile_name,
@@ -719,7 +845,7 @@ class ProfileConfigConvV11(ProfileConfigConv):
             http_profile["compression_profile"] = compression_profile
             app_profile["http_profile"] = http_profile
             # code to merge application profile count.
-            if self.object_merge_check:
+            if self.object_merge_check and not self.distinct_app_profile:
                 conv_utils.update_skip_duplicates(
                     app_profile, avi_config['ApplicationProfile'],
                     'app_profile', converted_objs, name, default_profile_name,
@@ -772,12 +898,15 @@ class ProfileConfigConvV11(ProfileConfigConv):
             explicit_tracking = profile.get("explicit-flow-migration", None)
             l4_profile = {"rl_profile": {
                 "client_ip_connections_rate_limit": {
-                    "explicit_tracking": (explicit_tracking == 'enabled')
+                    "explicit_tracking": (explicit_tracking == 'enabled'),
+                    "action": {
+                        "type": "RL_ACTION_NONE"
+                    }
                 }}
             }
             app_profile['dos_rl_profile'] = l4_profile
             # code to merge application profile count.
-            if self.object_merge_check:
+            if self.object_merge_check and not self.distinct_app_profile:
                 conv_utils.update_skip_duplicates(
                     app_profile, avi_config['ApplicationProfile'],
                     'app_profile', converted_objs, name, default_profile_name,
@@ -813,13 +942,14 @@ class ProfileConfigConvV11(ProfileConfigConv):
             insert_xff = profile.get('insert-xforwarded-for', 'disabled')
             insert_xff = True if insert_xff == 'enabled' else False
             http_profile['x_forwarded_proto_enabled'] = insert_xff
+            http_profile['xff_enabled'] = insert_xff
             header_size = profile.get('max-header-size',
                                       final.DEFAULT_MAX_HEADER)
             http_profile['client_max_header_size'] = \
                 int(header_size)/final.BYTES_IN_KB
             app_profile["http_profile"] = http_profile
             # code to merge application profile count.
-            if self.object_merge_check:
+            if self.object_merge_check and not self.distinct_app_profile:
                 conv_utils.update_skip_duplicates(
                     app_profile, avi_config['ApplicationProfile'],
                     'app_profile', converted_objs, name, default_profile_name,
@@ -990,13 +1120,15 @@ class ProfileConfigConvV11(ProfileConfigConv):
 
 class ProfileConfigConvV10(ProfileConfigConv):
     def __init__(self, f5_profile_attributes, object_merge_check, prefix,
-                 keypassphrase):
+                 keypassphrase, skip_pki, distinct_app_profile):
         """
 
         :param f5_profile_attributes: f5 profile attributes from yaml file.
         :param object_merge_check: flag for merging objects
         :param prefix: prefix for objects
         :param keypassphrase: keypassphrase yaml file location
+        :param skip_pki: PKI profile migration needs to be skipped
+        :param distinct_app_profile: Merge duplicates needs to be slipped
         """
         self.supported_types = f5_profile_attributes['Profile_supported_types']
         self.default_key = "defaults from"
@@ -1032,6 +1164,8 @@ class ProfileConfigConvV10(ProfileConfigConv):
         self.certkey_count = 0
         # Added prefix for objects
         self.prefix = prefix
+        self.skip_pki = skip_pki
+        self.distinct_app_profile = distinct_app_profile
 
     def convert_profile(self, profile, key, f5_config, profile_config,
                         avi_config, input_dir, user_ignore, tenant_ref,
@@ -1063,7 +1197,7 @@ class ProfileConfigConvV10(ProfileConfigConv):
         default_ignore = f5_config['profile'].get(default_profile_name, {})
         default_ignore.update(self.ignore_for_defaults)
         tenant, name = conv_utils.get_tenant_ref(name)
-        if not tenant_ref == 'admin':
+        if tenant_ref:
             tenant = tenant_ref
         old_name = name
         default_profile_name = profile_type
@@ -1142,7 +1276,7 @@ class ProfileConfigConvV10(ProfileConfigConv):
                 ca_file_name = ca_file_name.replace('\"', '').strip()
             else:
                 ca_file_name = None
-            if ca_file_name:
+            if ca_file_name and not self.skip_pki:
                 pki_profile = dict()
                 file_path = input_dir+os.path.sep+ca_file_name
                 pki_profile["name"] = name
@@ -1189,7 +1323,7 @@ class ProfileConfigConvV10(ProfileConfigConv):
             na_list = self.na_http
             indirect = self.indirect_http
             # code to merge application profile count.
-            if self.object_merge_check:
+            if self.object_merge_check and not self.distinct_app_profile:
                 conv_utils.update_skip_duplicates(
                     app_profile, avi_config['ApplicationProfile'],
                     'app_profile', converted_objs, name, default_profile_name,
@@ -1210,7 +1344,7 @@ class ProfileConfigConvV10(ProfileConfigConv):
                 tenant, 'tenant')
             app_profile['type'] = 'APPLICATION_PROFILE_TYPE_DNS'
             # code to merge application profile count.
-            if self.object_merge_check:
+            if self.object_merge_check and not self.distinct_app_profile:
                 conv_utils.update_skip_duplicates(
                     app_profile, avi_config['ApplicationProfile'],
                     'app_profile', converted_objs, name, default_profile_name,
@@ -1275,7 +1409,7 @@ class ProfileConfigConvV10(ProfileConfigConv):
                 converted_objs.append({'network_profile': ntwk_profile})
                 avi_config['NetworkProfile'].append(ntwk_profile)
             # code to merge application profile count.
-            if self.object_merge_check:
+            if self.object_merge_check and not self.distinct_app_profile:
                 conv_utils.update_skip_duplicates(
                     app_profile, avi_config['ApplicationProfile'],
                     'app_profile', converted_objs, name, default_profile_name,
@@ -1300,13 +1434,14 @@ class ProfileConfigConvV10(ProfileConfigConv):
             insert_xff = profile.get('insert xforwarded for', 'disabled')
             insert_xff = True if insert_xff == 'enabled' else False
             http_profile['x_forwarded_proto_enabled'] = insert_xff
+            http_profile['xff_enabled'] = insert_xff
             header_size = profile.get('max header size',
                                       final.DEFAULT_MAX_HEADER)
             http_profile['client_max_header_size'] = \
                 int(header_size)/final.BYTES_IN_KB
             app_profile["http_profile"] = http_profile
             # code to merge application profile count.
-            if self.object_merge_check:
+            if self.object_merge_check and not self.distinct_app_profile:
                 conv_utils.update_skip_duplicates\
                     (app_profile, avi_config['ApplicationProfile'],
                      'app_profile',converted_objs, name, default_profile_name,
@@ -1492,10 +1627,12 @@ class ProfileConfigConvV10(ProfileConfigConv):
         encpt_cookie = False if encpt_cookie == 'none' else True
         con_mltplxng = profile.get('oneconnect transformations', 'disabled')
         con_mltplxng = False if con_mltplxng == 'disabled' else True
-        http_profile['x_forwarded_proto_enabled'] = profile.get(
-            'insert xforwarded for', False)
-        http_profile['xff_alternate_name'] = profile.get(
-            'xff alternative names', None)
+        insert_xff = profile.get('insert xforwarded for', 'disabled')
+        insert_xff = True if insert_xff == 'enabled' else False
+        http_profile['x_forwarded_proto_enabled'] = insert_xff
+        xff_alt_names = profile.get('xff alternative names', None)
+        xff_alt_names = None if xff_alt_names == 'none' else xff_alt_names
+        http_profile['xff_alternate_name'] = xff_alt_names
         header_size = profile.get('max header size', final.DEFAULT_MAX_HEADER)
         http_profile['client_max_header_size'] = int(
             header_size)/final.BYTES_IN_KB
@@ -1579,11 +1716,11 @@ class ProfileConfigConvV10(ProfileConfigConv):
         header_erase = None if header_erase == 'none' else header_erase
 
         if header_erase:
-            header_erase = header_erase.replace('\"', '')
+            header_erase = header_erase.replace('\"', '').strip()
         header_insert = profile.get('header-insert', None)
         header_insert = None if header_insert == 'none' else header_insert
         if header_insert:
-            header_insert = header_insert.replace('\"', '')
+            header_insert = header_insert.replace('\"', '').strip()
 
         if header_erase or header_insert:
             rules = []
