@@ -5,28 +5,36 @@ import re
 import avi.migrationtools.f5_converter.converter_constants as final
 from avi.migrationtools.f5_converter.conversion_util import F5Util
 from avi.migrationtools.f5_converter.policy_converter import used_pools
+from avi.migrationtools.avi_migration_utils import update_count
 from pkg_resources import parse_version
 
 LOG = logging.getLogger(__name__)
 # Creating f5 object for util library.
 conv_utils = F5Util()
-used_policy=[]
+used_policy = list()
+used_app_profiles = list()
 
 class VSConfigConv(object):
     @classmethod
-    def get_instance(cls, version, f5_virtualservice_attributes, prefix, con_snatpool):
+    def get_instance(cls, version, f5_virtualservice_attributes, prefix,
+                     con_snatpool, custom_mappings, distinct_app_profile):
         """
 
         :param version:  version of f5 instance
         :param f5_virtualservice_attributes: yaml attribute file for object
         :param prefix: prefix for objects
         :param con_snatpool: flag for converting snat into  individual address
+        :param custom_mappings: custom config to migrate irules
         :return:
         """
         if version == '10':
-            return VSConfigConvV10(f5_virtualservice_attributes, prefix, con_snatpool)
+            return VSConfigConvV10(f5_virtualservice_attributes, prefix,
+                                   con_snatpool, custom_mappings,
+                                   distinct_app_profile)
         if version in ['11', '12']:
-            return VSConfigConvV11(f5_virtualservice_attributes, prefix, con_snatpool)
+            return VSConfigConvV11(f5_virtualservice_attributes, prefix,
+                                   con_snatpool, custom_mappings,
+                                   distinct_app_profile)
 
     def get_persist_ref(self, f5_vs):
         pass
@@ -35,9 +43,27 @@ class VSConfigConv(object):
                                sys_dict):
         pass
 
+    def create_partition_mapping(self, f5_vs, vs_name):
+        dest = f5_vs['destination']
+        tenant, vs_name = conv_utils.get_tenant_ref(vs_name)
+        if not tenant:
+            tenant = 'admin'
+        if not tenant in dest:
+            vip = dest
+        else:
+            vip = dest.split('/')[-1]
+        p_mapping = {
+            vip: {
+                'vs_name': vs_name,
+                'partition': tenant
+            }
+        }
+        return p_mapping
+
     def convert(self, f5_config, avi_config, vs_state, user_ignore, tenant,
                 cloud_name, controller_version, merge_object_mapping, sys_dict,
-                vrf=None, segroup=None):
+                vrf=None, segroup=None, partition_mapping=None,
+                reuse_http_policy=False):
         """
 
         :param f5_config: Parsed f5 config dict
@@ -49,6 +75,8 @@ class VSConfigConv(object):
         :param controller_version: AVI controller version
         :param merge_object_mapping: flag for merge object
         :param sys_dict: baseline profile dict
+        :param vrf: vrf user input to put vrf ref in VS object
+        :param segroup: segroup user input to put se-group ref in VS object
         :return:
         """
         f5_snat_pools = f5_config.get("snatpool", {})
@@ -75,32 +103,37 @@ class VSConfigConv(object):
                     conv_utils.add_status_row('virtual', None, vs_name,
                                               final.STATUS_SKIPPED, msg)
                     continue
-                vs_obj = self.convert_vs(vs_name, f5_vs, vs_state, avi_config,
-                                         f5_snat_pools, user_ignore, tenant,
-                                         cloud_name, controller_version,
-                                         merge_object_mapping, sys_dict, vrf,
-                                         segroup)
+                mapping = self.create_partition_mapping(f5_vs, vs_name)
+                partition_mapping.update(mapping)
+
+                vs_obj = self.convert_vs(
+                    vs_name, f5_vs, vs_state, avi_config, f5_snat_pools,
+                    user_ignore, tenant, cloud_name, controller_version,
+                    merge_object_mapping, sys_dict, f5_config, vrf, 
+                    reuse_http_policy)
                 if vs_obj:
                     if segroup:
-                        segroup_ref = conv_utils.get_object_ref(segroup,
-                                                    'serviceenginegroup',
-                                                    tenant=tenant,
-                                                    cloud_name=cloud_name)
-                        vs_obj['segroup_ref'] = segroup_ref
+                        segroup_ref = conv_utils.get_object_ref(
+                            segroup, 'serviceenginegroup', tenant=tenant,
+                            cloud_name=cloud_name)
+                        vs_obj['se_group_ref'] = segroup_ref
                     avi_config['VirtualService'].append(vs_obj)
                     LOG.debug("Conversion successful for VS: %s" % vs_name)
             except:
+                update_count('error')
                 LOG.error("Failed to convert VS: %s" % vs_name, exc_info=True)
             # Added call to get the progress.
             msg = "virtualservice conversion started..."
-            conv_utils.print_progress_bar(progressbar_count, total_size, msg,
-                             prefix='Progress', suffix='')
+            conv_utils.print_progress_bar(
+                progressbar_count, total_size, msg, prefix='Progress',
+                suffix='')
         LOG.debug("Converted %s VS" % len(avi_config['VirtualService']))
         f5_config.pop("virtual", {})
 
     def convert_vs(self, vs_name, f5_vs, vs_state, avi_config, snat_config,
                    user_ignore, tenant_ref, cloud_name, controller_version,
-                   merge_object_mapping, sys_dict, vrf=None, segroup=None):
+                   merge_object_mapping, sys_dict, f5_config, vrf=None,
+                   reuse_http_policy=False):
         """
 
         :param vs_name: name of virtual service.
@@ -114,11 +147,13 @@ class VSConfigConv(object):
         :param controller_version: AVI controller version
         :param merge_object_mapping: Flag to merge object
         :param sys_dict: Baseline dict
+        :param vrf: vrf user input to put vrf ref in VS object
         :return:
         """
+        needs_review = False
         tenant, vs_name = conv_utils.get_tenant_ref(vs_name)
         tenant_name = tenant
-        if not tenant_ref == 'admin':
+        if tenant_ref:
             tenant = tenant_ref
         # Added prefix for objects
         if self.prefix:
@@ -131,18 +166,28 @@ class VSConfigConv(object):
         if enabled:
             enabled = False if "disabled" in f5_vs.keys() else True
         profiles = f5_vs.get("profiles", {})
-        ssl_vs, ssl_pool = conv_utils.get_vs_ssl_profiles(profiles, avi_config,
-                                    self.prefix, merge_object_mapping, sys_dict)
+        ssl_vs, ssl_pool = conv_utils.get_vs_ssl_profiles(
+            profiles, avi_config, self.prefix, merge_object_mapping, sys_dict,
+            f5_config)
+
+        if (ssl_vs and len(ssl_vs) > 1) or (ssl_pool and len(ssl_pool)> 1):
+            needs_review = True
+
         oc_prof = False
         for prof in profiles:
-            if prof in avi_config.get('OneConnect', []):
+            prof_name = prof.split('/')[-1] if '/' in prof else prof
+            if prof_name in avi_config.get('OneConnect', []):
                 oc_prof = True
         enable_ssl = False
         if ssl_vs:
             enable_ssl = True
-        app_prof, f_host, realm, app_pol_name = conv_utils.get_vs_app_profiles(
+        app_prof_conf = conv_utils.get_vs_app_profiles(
             profiles, avi_config, tenant, self.prefix, oc_prof, enable_ssl,
             merge_object_mapping, sys_dict)
+        app_prof = app_prof_conf.get('app_prof', None)
+        f_host = app_prof_conf.get('f_host', None)
+        realm = app_prof_conf.get('realm', None)
+        app_pol_name = app_prof_conf.get('app_pol_name', None)
 
         if not app_prof:
             msg = ('Profile type not supported by Avi Skipping VS : %s'
@@ -152,8 +197,8 @@ class VSConfigConv(object):
                                       final.STATUS_SKIPPED, msg)
             return None
 
-        ntwk_prof = conv_utils.get_vs_ntwk_profiles(profiles, avi_config,
-                                    self.prefix, merge_object_mapping, sys_dict)
+        ntwk_prof = conv_utils.get_vs_ntwk_profiles(
+            profiles, avi_config, self.prefix, merge_object_mapping, sys_dict)
 
         # If one connect profile is not assigned to f5 VS and avi app profile
         # assigned to VS has connection_multiplexing_enabled value True then
@@ -167,19 +212,28 @@ class VSConfigConv(object):
         app_prof_type = None
         if app_prof_obj:
             app_prof_type = app_prof_obj[0].get('type')
+        else:
+            if app_name == 'System-L4-Application':
+                app_prof_type = 'APPLICATION_PROFILE_TYPE_L4'
+            elif app_name in ['System-HTTP', 'System-Secure-HTTP']:
+                app_prof_type = 'APPLICATION_PROFILE_TYPE_HTTP'
+            elif app_name == 'System-SSL-Application':
+                app_prof_type = 'APPLICATION_PROFILE_TYPE_SSL'
+
         if app_prof_type == 'APPLICATION_PROFILE_TYPE_HTTP':
             cme = app_prof_obj[0]['http_profile'].get(
                 'connection_multiplexing_enabled', False)
-        if not (cme or oc_prof):
+        if app_prof_obj and not (cme and oc_prof):
             # Check if already cloned profile present
-            app_prof_cmd = [obj for obj in (sys_dict['ApplicationProfile'] +
-                                        avi_config['ApplicationProfile']) if
+            app_prof_cmd = [obj for obj in (
+                    sys_dict['ApplicationProfile'] +
+                    avi_config['ApplicationProfile']) if
                             obj['name'] == '%s-cmd' % app_name]
             if app_prof_cmd:
                 app_name = app_prof_cmd[0]['name']
-                app_prof[0] = conv_utils.get_object_ref(app_name,
-                      'applicationprofile', tenant=conv_utils.get_name(
-                                                 app_prof_cmd[0]['tenant_ref']))
+                app_prof[0] = conv_utils.get_object_ref(
+                    app_name, 'applicationprofile',
+                    tenant=conv_utils.get_name(app_prof_cmd[0]['tenant_ref']))
             else:
                 app_prof_cmd = copy.deepcopy(app_prof_obj[0])
                 app_prof_cmd['name'] = '%s-cmd' % app_prof_cmd['name']
@@ -188,9 +242,9 @@ class VSConfigConv(object):
                         'connection_multiplexing_enabled'] = False
                 avi_config['ApplicationProfile'].append(app_prof_cmd)
                 app_name = app_prof_cmd['name']
-                app_prof[0] = conv_utils.get_object_ref(app_name,
-                     'applicationprofile', tenant=conv_utils.get_name(
-                                                 app_prof_cmd['tenant_ref']))
+                app_prof[0] = conv_utils.get_object_ref(
+                    app_name, 'applicationprofile',
+                    tenant=conv_utils.get_name(app_prof_cmd['tenant_ref']))
         destination = f5_vs.get("destination", None)
         d_tenant, destination = conv_utils.get_tenant_ref(destination)
         # if destination is not present then skip vs.
@@ -216,6 +270,8 @@ class VSConfigConv(object):
         is_pool_group = False
         if pool_ref:
             p_tenant, pool_ref = conv_utils.get_tenant_ref(pool_ref)
+            if tenant_ref:
+                p_tenant = tenant_ref
             persist_ref = self.get_persist_ref(f5_vs)
             avi_persistence = avi_config['ApplicationPersistenceProfile']
             syspersist = sys_dict['ApplicationPersistenceProfile']
@@ -225,15 +281,15 @@ class VSConfigConv(object):
                 persist_ref = conv_utils.get_tenant_ref(persist_ref)[1]
                 if self.prefix:
                     persist_ref = '{}-{}'.format(self.prefix, persist_ref)
-                persist_profile_objs = [ob for ob in syspersist if ob['name'] ==
-                                       merge_object_mapping[
-                                       'app_per_profile'].get(persist_ref)] or \
-                                       [obj for obj in avi_persistence if
-                                       (obj["name"] == persist_ref or
-                                        persist_ref in obj.get(
-                                       "dup_of", []))]
-                persist_type = persist_profile_objs[0]['persistence_type'] if\
-                                persist_profile_objs else None
+                persist_profile_objs = (
+                        [ob for ob in syspersist if ob['name'] ==
+                         merge_object_mapping['app_per_profile'].get(
+                             persist_ref)] or
+                        [obj for obj in avi_persistence if
+                         (obj["name"] == persist_ref or persist_ref in obj.get(
+                             "dup_of", []))])
+                persist_type = (persist_profile_objs[0]['persistence_type'] if
+                                persist_profile_objs else None)
             # Pool cloned if controller version < 17.1.6 or VS has non http
             # cookie persistence or app profile type is different and poolgroup
             # cloned
@@ -266,29 +322,41 @@ class VSConfigConv(object):
                 if is_pool_group:
                     pool_updated, persist_type = \
                         conv_utils.update_pool_group_for_persist(
-                        avi_config, pool_ref, persist_ref, hash_profiles,
-                        avi_persistence, tenant, merge_object_mapping,
-                        syspersist, app_prof_type)
+                            avi_config, pool_ref, persist_ref, hash_profiles,
+                            avi_persistence, tenant, merge_object_mapping,
+                            syspersist, app_prof_type)
                 else:
                     pool_updated, persist_type = \
                         conv_utils.update_pool_for_persist(
-                        avi_config['Pool'], pool_ref, persist_ref,
-                        hash_profiles, avi_persistence, tenant,
-                        merge_object_mapping, syspersist, app_prof_type)
+                            avi_config['Pool'], pool_ref, persist_ref,
+                            hash_profiles, avi_persistence, tenant,
+                            merge_object_mapping, syspersist, app_prof_type)
 
                 if not pool_updated:
                     skipped.append("persist")
                     LOG.warning(
                         "persist profile %s not found for vs:%s" %
                         (persist_ref, vs_name))
-            if oc_prof and not ssl_vs and persist_type == \
-                    'PERSISTENCE_TYPE_TLS':
+            if (oc_prof and not ssl_vs and
+                    persist_type == 'PERSISTENCE_TYPE_TLS' or
+                    persist_type == 'PERSISTENCE_TYPE_TLS'
+                    and not enable_ssl):
                 msg = ("Skipped VS : '%s' Secure persistence is applicable only"
                        " if SSL is enabled for Virtual Service" % vs_name)
                 LOG.warning(msg)
                 conv_utils.add_status_row('virtual', None, vs_name,
                                           final.STATUS_SKIPPED, msg)
                 return
+            # TODO: Followiong condition to be removed after controller adds
+            # TODO: support for PERSISTENCE_TYPE_TLS for SSL VS
+            elif (persist_type == 'PERSISTENCE_TYPE_TLS' and
+                  app_prof_type == 'APPLICATION_PROFILE_TYPE_SSL'):
+                msg = ("Skipped VS : '%s' Only client-ip persistence is "
+                       "applicable for SSL VS" % vs_name)
+                LOG.warning(msg)
+                conv_utils.add_status_row('virtual', None, vs_name,
+                                          final.STATUS_SKIPPED, msg)
+                return
             if f_host:
                 conv_utils.update_pool_for_fallback(
                     f_host, avi_config['Pool'], pool_ref)
@@ -300,19 +368,27 @@ class VSConfigConv(object):
             vs_name += '-needs-ipv6-ip'
             ip_addr = ".".join(map(str, (
                 random.randint(0, 255) for _ in range(4))))
+
+        if app_prof_obj:
+            if self.distinct_app_profile and app_prof[0] in used_app_profiles:
+                app_prof[0] = conv_utils.clone_app_profile_for_vs(
+                    app_prof[0], app_prof_obj[0], vs_name, tenant, avi_config)
+            used_app_profiles.append(app_prof[0])
+
         # VIP object for virtual service
         vip = {
             'ip_address': {
                 'addr': ip_addr,
                 'type': 'V4'
             },
-            'vip_id': 0
+            'vip_id': '1'
         }
         vs_obj = {
             'name': vs_name,
             'description': description,
             'type': 'VS_TYPE_NORMAL',
-            'enabled': enabled,
+            'enabled': True,
+            'traffic_enabled': enabled,
             'cloud_ref': conv_utils.get_object_ref(
                 cloud_name, 'cloud', tenant=tenant),
             'services': services_obj,
@@ -324,8 +400,7 @@ class VSConfigConv(object):
         if vrf:
             vrf_ref = conv_utils.get_object_ref(vrf, 'vrfcontext',
                                                 tenant=tenant_name,
-                                                cloud_name=cloud_ref)
-
+                                                cloud_name=cloud_name)
         if vrf_ref:
             vs_obj['vrf_context_ref'] = vrf_ref
             # Added code for assigning VS's vrf ref to poolgroup/pool having no
@@ -349,72 +424,59 @@ class VSConfigConv(object):
         # Policy tracking starts from here
         vs_policies = [app_pol_name] if app_pol_name else []
         vs_ds_rules = None
+        vs_ds = list()
+        nw_policy = None
+        converted_rules = list()
         if 'rules' in f5_vs:
             if isinstance(f5_vs['rules'], basestring):
                 vs_ds_rules = [conv_utils.get_tenant_ref(f5_vs['rules'])[1]]
             else:
                 vs_ds_rules = [conv_utils.get_tenant_ref(name)[1] for name in
                                f5_vs['rules'].keys()]
-            for index, rule in enumerate(vs_ds_rules):
-                # converted _sys_https_redirect data script to rule in
-                # http policy
-                if rule == '_sys_https_redirect':
-                    # Added prefix for objects
-                    if self.prefix:
-                        policy_name = self.prefix + '-' + rule + '-' + vs_name
-                    else:
-                        policy_name = rule + '-' + vs_name
-                    policy = {
-                        "name": policy_name,
-                        "http_request_policy": {
-                            "rules": [
-                                {
-                                    "index": 1,
-                                    "redirect_action": {
-                                        "keep_query": True,
-                                        "status_code":
-                                            "HTTP_REDIRECT_STATUS_CODE_302",
-                                        "protocol": "HTTPS",
-                                        "port": 443
-                                    },
-                                    "enable": True,
-                                    "name": policy_name + "-Redirect",
-                                    "match": {
-                                        "protocol": {
-                                            "protocols": "HTTP",
-                                            "match_criteria": "IS_IN"
-                                        }
-                                    }
-                                }
-                            ]
-                        },
-                        'tenant_ref': conv_utils.get_object_ref(tenant,
-                                                                'tenant'),
-                        "is_internal_policy": False
+
+            vs_ds, req_policies, nw_policy, converted_rules = (
+                conv_utils.convert_irules(
+                    vs_ds_rules, self.rule_config, avi_config, self.prefix,
+                    vs_name, tenant, reuse_http_policy))
+            vs_policies = vs_policies + req_policies
+        if vs_ds:
+            vs_datascripts = []
+            index = 1
+            for ds in vs_ds:
+                vs_datascripts.append(
+                    {
+                        "index": index,
+                        "vs_datascript_set_ref": conv_utils.get_object_ref(
+                            ds, 'vsdatascriptset', tenant=tenant)
                     }
-                    vs_policies.append(policy_name)
-                    avi_config['HTTPPolicySet'].append(policy)
+                )
+                index += 1
+            vs_obj['vs_datascripts'] = vs_datascripts
+
         if 'policies' in f5_vs:
             if isinstance(f5_vs['policies'], basestring):
-                vs_policies.extend(['%s-%s' % (self.prefix,
-                               conv_utils.get_tenant_ref(f5_vs['policies'])[1])
-                               if self.prefix else conv_utils.get_tenant_ref(
-                               f5_vs['policies'])[1]])
+                vs_policies.extend(['%s-%s' % (
+                    self.prefix, conv_utils.get_tenant_ref(
+                        f5_vs['policies'])[1]) if self.prefix else
+                                    conv_utils.get_tenant_ref(
+                                        f5_vs['policies'])[1]])
             else:
-                vs_policies.extend(['%s-%s' % (self.prefix,
-                               conv_utils.get_tenant_ref(name)[1]) if
-                               self.prefix else conv_utils.get_tenant_ref(
-                               name)[1] for name in f5_vs['policies'].keys()])
+                vs_policies.extend(['%s-%s' % (
+                    self.prefix, conv_utils.get_tenant_ref(name)[1]) if
+                                    self.prefix else conv_utils.get_tenant_ref(
+                    name)[1] for name in f5_vs['policies'].keys()])
+        if app_prof_obj and 'HTTPPolicySet' in app_prof_obj[0]:
+            vs_policies.append(app_prof_obj[0]['HTTPPolicySet'])
         if vs_policies:
             self.get_policy_vs(vs_policies, avi_config, vs_name, tenant,
-                               cloud_name, vs_obj)
+                               cloud_name, vs_obj, reuse_http_policy)
         p_ref = None
         if is_pool_group:
-            p_ref = conv_utils.get_object_ref(pool_ref, 'poolgroup',
-                                               tenant=p_tenant)
+            p_ref = conv_utils.get_object_ref(
+                pool_ref, 'poolgroup', tenant=p_tenant)
         elif pool_ref:
-            p_ref = conv_utils.get_object_ref(pool_ref, 'pool',
-                                              tenant=p_tenant)
+            p_ref = conv_utils.get_object_ref(
+                pool_ref, 'pool', tenant=p_tenant)
         if p_ref and used_pools.get(p_ref):
             not_same = [pol_obj for pol_obj in used_pools[p_ref] if pol_obj
                         not in vs_policies]
@@ -423,15 +485,15 @@ class VSConfigConv(object):
                     LOG.debug('Pool group %s attached to vs %s is shared '
                               'with policy %s of another vs, hence cloned',
                               pool_ref, vs_name, str(not_same))
-                    pool_ref = conv_utils.clone_pool_group(pool_ref, vs_name,
-                                avi_config, False, p_tenant,
-                                cloud_name=cloud_name)
+                    pool_ref = conv_utils.clone_pool_group(
+                        pool_ref, vs_name, avi_config, False, p_tenant,
+                        cloud_name=cloud_name)
                 else:
                     LOG.debug('Pool %s attached to vs %s is shared with '
                               'policy %s of another vs, hence cloned', pool_ref,
                               vs_name, str(not_same))
-                    pool_ref = conv_utils.clone_pool(pool_ref, vs_name,
-                                    avi_config['Pool'], False, p_tenant)
+                    pool_ref = conv_utils.clone_pool(
+                        pool_ref, vs_name, avi_config['Pool'], False, p_tenant)
         if is_pool_group:
             vs_obj['pool_group_ref'] = conv_utils.get_object_ref(
                 pool_ref, 'poolgroup', tenant=tenant, cloud_name=cloud_name)
@@ -449,7 +511,15 @@ class VSConfigConv(object):
         rate_limit = int(f5_vs.get('rate-limit', '0'))
         if rate_limit > 0:
             vs_obj["connections_rate_limit"] = {
-                "count": rate_limit
+                "count": rate_limit,
+                'explicit_tracking': False,
+                'period': 1,
+                'action': {
+                    'status_code': 'HTTP_LOCAL_RESPONSE_STATUS_CODE_429',
+                    'type': "RL_ACTION_NONE"
+                },
+                'burst_sz': 0,
+                'fine_grain': False
             }
 
         if realm:
@@ -470,13 +540,27 @@ class VSConfigConv(object):
             if len(parts) > 1:
                 mask = parts[1]
             policy_name = ('vs-%s-ns' % vs_name)
+            if self.prefix:
+                policy_name = '%s-%s' % (self.prefix, policy_name)
             policy = conv_utils.create_network_security_rule(
                 policy_name, parts[0], mask, tenant)
-            avi_config['NetworkSecurityPolicy'].append(policy)
-            vs_obj['network_security_policy_ref'] = conv_utils.get_object_ref(
-                policy_name, 'networksecuritypolicy', tenant=tenant)
 
-        # Checking snat conversion flag and snat info for creating snat ip object
+            if nw_policy:
+                old_policy = [obj for obj in avi_config['NetworkSecurityPolicy']
+                              if obj['name'] == nw_policy][0]
+                policy['rules'][0]['index'] = 2
+                policy['rules'][0]['name'] = 'Rule 2'
+                old_policy['rules'].append(policy['rules'][0])
+            else:
+                avi_config['NetworkSecurityPolicy'].append(policy)
+                nw_policy = policy_name
+
+        if nw_policy:
+            vs_obj['network_security_policy_ref'] = conv_utils.get_object_ref(
+                nw_policy, 'networksecuritypolicy', tenant=tenant)
+
+        # Checking snat conversion flag and snat info for creating
+        # snat ip object
         snat = f5_vs.get("source-address-translation", {})
         snat_pool_name = snat.get("pool", f5_vs.get("snatpool", None))
         snat_pool = snat_config.pop(snat_pool_name, None)
@@ -512,8 +596,9 @@ class VSConfigConv(object):
             if ssl_vs[0]["cert"]:
                 vs_obj['ssl_key_and_certificate_refs'] = [ssl_vs[0]["cert"]]
             if ssl_vs[0]["pki"] and app_name != "http":
-                app_profiles = [obj for obj in (sys_dict['ApplicationProfile'] +
-                                        avi_config['ApplicationProfile'])
+                app_profiles = [obj for obj in (
+                        sys_dict['ApplicationProfile'] +
+                        avi_config['ApplicationProfile'])
                                 if obj['name'] == app_name]
                 if app_profiles[0]["type"] == \
                         'APPLICATION_PROFILE_TYPE_HTTP':
@@ -526,12 +611,13 @@ class VSConfigConv(object):
         if vs_obj['application_profile_ref']:
             application_profile_obj = \
                 [obj for obj in (sys_dict['ApplicationProfile'] +
-                avi_config['ApplicationProfile']) if obj['name'] == app_name]
+                                 avi_config['ApplicationProfile'])
+                 if obj['name'] == app_name]
             if application_profile_obj and application_profile_obj[0]['type'] \
                     == 'APPLICATION_PROFILE_TYPE_L4':
-                if not vs_obj.get('pool_ref',vs_obj.get('pool_group_ref')):
-                    msg = ("Failed to convert L4 VS dont have "
-                              "pool or pool group ref: %s" % vs_name)
+                if not vs_obj.get('pool_ref', vs_obj.get('pool_group_ref')):
+                    msg = ("Failed to convert L4 VS dont have pool or pool "
+                           "group ref: %s" % vs_name)
                     LOG.debug(msg)
                     conv_utils.add_status_row('virtual', None,
                                               vs_name,
@@ -552,8 +638,9 @@ class VSConfigConv(object):
                                       if val in user_ignore]
 
         if vs_ds_rules:
-            skipped_rules = [rule for rule in vs_ds_rules
-                             if rule != '_sys_https_redirect']
+            vs_ds.append('_sys_https_redirect')
+            skipped_rules = [rule for rule in vs_ds_rules if rule not in
+                             converted_rules]
             if skipped_rules:
                 skipped.append('rules: %s' % skipped_rules)
         conv_status['na_list'] = [val for val in skipped if
@@ -565,14 +652,14 @@ class VSConfigConv(object):
         if skipped:
             status = final.STATUS_PARTIAL
         conv_status['status'] = status
-
+        review_flag = 'Yes' if needs_review else None
         conv_utils.add_conv_status('virtual', None, vs_name,
-                                   conv_status, vs_obj)
+                                   conv_status, vs_obj, review_flag)
 
         return vs_obj
 
     def get_policy_vs(self, vs_policies, avi_config, vs_name, tenant,
-                      cloud_name, vs_obj):
+                      cloud_name, vs_obj, reuse_http_policy=False):
         """
         This method gets all the policy attached to vs, also clone it if
         required
@@ -588,22 +675,33 @@ class VSConfigConv(object):
             policy_obj = [ob for ob in avi_config['HTTPPolicySet'] if ob[
                 'name'] == pol_name]
             if policy_obj:
-                if pol_name in used_policy:
+                if reuse_http_policy:
+                    pass
+                elif pol_name in used_policy:
                     LOG.debug('Cloning the policy %s for vs %s',
                               pol_name, vs_name)
                     clone_policy = conv_utils.clone_http_policy_set(
                         policy_obj[0], vs_name, avi_config, tenant, cloud_name)
                     pol_name = clone_policy['name']
-                    avi_config['HTTPPolicySet'].append(clone_policy)
-                    LOG.debug('Policy cloned %s for vs %s', pol_name,
-                              vs_name)
-                used_policy.append(pol_name)
+                    pol_tenant = conv_utils.get_name(clone_policy['tenant_ref'])
+                    if pol_tenant == tenant:
+                        avi_config['HTTPPolicySet'].append(clone_policy)
+                        LOG.debug('Policy cloned %s for vs %s', pol_name,
+                                  vs_name)
+                    else:
+                        LOG.debug('Policy with different tenant not '
+                                   'supported  %s for vs %s', pol_name,
+                                  vs_name)
+                        continue
+                if pol_name not in used_policy:
+                    used_policy.append(pol_name)
                 pol = {
                     'index': 11,
                     'http_policy_set_ref':
-                        conv_utils.get_object_ref(pol_name, 'httppolicyset',
-                                                  tenant=conv_utils.get_name(
-                                                  policy_obj[0]['tenant_ref']))
+                        conv_utils.get_object_ref(
+                            pol_name, 'httppolicyset',
+                            tenant=conv_utils.get_name(
+                                policy_obj[0]['tenant_ref']))
                 }
                 if not vs_obj.get('http_policies'):
                     vs_obj['http_policies'] = []
@@ -615,12 +713,14 @@ class VSConfigConv(object):
 
 
 class VSConfigConvV11(VSConfigConv):
-    def __init__(self, f5_virtualservice_attributes, prefix, con_snatpool):
+    def __init__(self, f5_virtualservice_attributes, prefix, con_snatpool,
+                 custom_mappings, distinct_app_profile):
         """
 
         :param f5_virtualservice_attributes: yaml attribute file for object
         :param prefix: prefix for object
         :param con_snatpool: flag for snat conversion
+        :param custom_mappings: custom config to migrate irules
         """
         self.supported_attr = f5_virtualservice_attributes['VS_supported_attr']
         self.ignore_for_value = \
@@ -634,6 +734,10 @@ class VSConfigConvV11(VSConfigConv):
         self.prefix = prefix
         # Added flag for snat conversion
         self.con_snatpool = con_snatpool
+        self.rule_config = custom_mappings.get(
+            final.RULE_CUSTOM_KEY, dict()
+        ) if custom_mappings else dict()
+        self.distinct_app_profile = distinct_app_profile
 
     def get_persist_ref(self, f5_vs):
         """
@@ -661,20 +765,21 @@ class VSConfigConvV11(VSConfigConv):
         port_translate = f5_vs.get('translate-port', None)
         if port_translate:
             if port_translate == 'disabled':
-                conv_utils.update_pool_for_service_port(avi_config['Pool'],
-                                        pool_ref, avi_config['HealthMonitor'],
-                                                    sys_dict['HealthMonitor'])
+                conv_utils.update_pool_for_service_port(
+                    avi_config['Pool'], pool_ref, avi_config['HealthMonitor'],
+                    sys_dict['HealthMonitor'])
             elif port_translate == 'enabled':
                 return
 
-
 class VSConfigConvV10(VSConfigConv):
-    def __init__(self, f5_virtualservice_attributes, prefix, con_snatpool):
+    def __init__(self, f5_virtualservice_attributes, prefix, con_snatpool,
+                 custom_mappings, distinct_app_profile):
         """
 
         :param f5_virtualservice_attributes: yaml attribute file for object
         :param prefix: prefix for object
         :param con_snatpool: flag for snat conversion
+        :param custom_mappings: custom config to migrate irules
         """
         self.supported_attr = f5_virtualservice_attributes['VS_supported_attr']
         self.ignore_for_value = \
@@ -688,6 +793,10 @@ class VSConfigConvV10(VSConfigConv):
         self.prefix = prefix
         # Added flag for snat conversion
         self.con_snatpool = con_snatpool
+        self.rule_config = custom_mappings.get(
+            final.RULE_CUSTOM_KEY, dict()
+        ) if custom_mappings else dict()
+        self.distinct_app_profile = distinct_app_profile
 
     def get_persist_ref(self, f5_vs):
         persist_ref = f5_vs.get("persist", None)
@@ -708,8 +817,8 @@ class VSConfigConvV10(VSConfigConv):
         port_translate = f5_vs.get('translate service', None)
         if port_translate:
             if port_translate == 'disabled':
-                conv_utils.update_pool_for_service_port(avi_config['Pool'],
-                                        pool_ref, avi_config['HealthMonitor'],
-                                                    sys_dict['HealthMonitor'])
+                conv_utils.update_pool_for_service_port(
+                    avi_config['Pool'], pool_ref, avi_config['HealthMonitor'],
+                    sys_dict['HealthMonitor'])
             elif port_translate == 'enabled':
-                return
+                return
\ No newline at end of file
