@@ -7,7 +7,7 @@ from avi.migrationtools.netscaler_converter.ns_constants \
     import (STATUS_EXTERNAL_MONITOR, STATUS_MISSING_FILE)
 from avi.migrationtools.netscaler_converter.ns_util import NsUtil
 from pkg_resources import parse_version
-
+from avi.migrationtools.avi_migration_utils import update_count
 
 
 LOG = logging.getLogger(__name__)
@@ -76,6 +76,7 @@ class MonitorConverter(object):
         total_size = len(ns_monitors.keys())
         count = 0
         print "Converting Monitors..."
+        self.add_ns_default_monitors(avi_config)
         for name in ns_monitors.keys():
             count = count + 1
             ns_monitor = ns_monitors.get(name)
@@ -89,6 +90,8 @@ class MonitorConverter(object):
                      ns_monitor, input_dir, netscalar_command,
                     ns_monitor_complete_command)
                 if not avi_monitor:
+                    ns_util.print_progress_bar(count, total_size, msg,
+                                               prefix='Progress', suffix='')
                     continue
 
                 avi_monitor['name'] = '%s-%s' % (avi_monitor['name'], 'dummy')
@@ -105,12 +108,16 @@ class MonitorConverter(object):
                 ns_util.add_status_row(ns_monitor['line_no'], netscalar_command,
                     name, ns_monitor_complete_command, STATUS_EXTERNAL_MONITOR,
                     msg)
+                ns_util.print_progress_bar(count, total_size, msg,
+                                           prefix='Progress', suffix='')
                 continue
 
             avi_monitor = self.convert_monitor(
                  ns_monitor, input_dir, netscalar_command,
                  ns_monitor_complete_command)
             if not avi_monitor:
+                ns_util.print_progress_bar(count, total_size, msg,
+                                           prefix='Progress', suffix='')
                 continue
 
             # Add summery of this lb vs in CSV/report
@@ -254,14 +261,23 @@ class MonitorConverter(object):
             elif mon_type == 'HTTPS':
                 avi_monitor["type"] = "HEALTH_MONITOR_HTTPS"
                 send = ns_monitor.get('httpRequest', None)
+                if send is None:
+                    send = ns_monitor.get('send', None)
                 if send:
                     send = send.replace('"', '')
                     # Removed \\ from send.
                     if '\\' in send:
                         send = send.replace('\\', '"')
-                resp_code = ns_monitor.get('respCode', None)
+                resp_code = ns_monitor.get('respCode', 'any')
                 if resp_code:
                     resp_code = ns_util.get_avi_resp_code(resp_code)
+
+                response = ns_monitor.get('recv', None)
+                if response:
+                    response = response.replace('"', '')
+                    # Removed \\ from response.
+                    if '\\' in response:
+                        response = response.replace('\\', '"')
                 # TODO: Remove this after all the clients are moved to 
                 # 17 version and above
                 if parse_version(self.controller_version) >= parse_version(
@@ -269,7 +285,8 @@ class MonitorConverter(object):
                     avi_monitor["https_monitor"] = {
                         "http_request": send,
                         "http_response_code": resp_code,
-                        "ssl_attributes": ssl_attributes
+                        "ssl_attributes": ssl_attributes,
+                        "http_response": response
                     }
                 if parse_version(self.controller_version) >= parse_version(
                         '17.1.6'):
@@ -356,5 +373,28 @@ class MonitorConverter(object):
 
             LOG.debug('Successfully converted monitor %s' % mon_name)
         except:
+            update_count('error')
             LOG.error('Error converting monitor %s', exc_info=True)
         return avi_monitor
+
+    def add_ns_default_monitors(self, avi_config):
+        tcp_default = {
+            'name': 'tcp-default',
+            'receive_timeout': 2,
+            'failed_checks': 2,
+            'send_interval': 5,
+            'successful_checks': 1,
+            'type': 'HEALTH_MONITOR_TCP',
+            'tenant_ref': self.tenant_ref
+        }
+        avi_config['HealthMonitor'].append(tcp_default)
+        ping_default = {
+            'name': 'ping-default',
+            'receive_timeout': 2,
+            'failed_checks': 2,
+            'send_interval': 5,
+            'successful_checks': 1,
+            'type': 'HEALTH_MONITOR_PING',
+            'tenant_ref': self.tenant_ref
+        }
+        avi_config['HealthMonitor'].append(ping_default)
\ No newline at end of file
