@@ -18,13 +18,14 @@ from avi.migrationtools.netscaler_converter.profile_converter import \
 from avi.migrationtools.avi_converter import AviConverter
 from avi.migrationtools.netscaler_converter import ns_util as nsu
 from avi.migrationtools.netscaler_converter.ns_util import NsUtil
+from avi.migrationtools.avi_migration_utils import update_count
 
 
 LOG = logging.getLogger(__name__)
 # Creating object for util library.
 ns_util = NsUtil()
 
-def convert(meta, ns_config_dict, tenant_name, cloud_name, version, output_dir,
+def convert(ns_config_dict, tenant_name, cloud_name, version, output_dir,
             input_dir, skipped_cmds, vs_state, object_merge_check,report_name,
             prefix, vs_name_dict, profile_path, redirect, key_passphrase=None,
             user_ignore={}, vs_level_status=False, vrf=None, segroup=None):
@@ -59,7 +60,6 @@ def convert(meta, ns_config_dict, tenant_name, cloud_name, version, output_dir,
         # call meta from super class
         avi_config = dict()
         sys_dict = dict()
-        avi_config['META'] = meta  # avi_obj.meta(tenant_name, version)
 
         merge_object_type = ['ApplicationProfile', 'NetworkProfile',
                              'SSLProfile', 'PKIProfile',
@@ -206,6 +206,7 @@ def convert(meta, ns_config_dict, tenant_name, cloud_name, version, output_dir,
                 print 'Total Objects of %s : %s' % (key, len(avi_config[key]))
 
     except:
+        update_count('warning')
         LOG.error('Error in config conversion', exc_info=True)
 
     return avi_config
