@@ -3,18 +3,20 @@ import argparse
 import logging
 import os
 import sys
-import json
+
 import yaml
+
 import avi.migrationtools
 import avi.migrationtools.netscaler_converter.netscaler_config_converter \
     as ns_conf_converter
 import avi.migrationtools.netscaler_converter.netscaler_parser as ns_parser
 import avi.migrationtools.netscaler_converter.scp_util as scp_util
-from avi.migrationtools.ansible.ansible_config_converter import AviAnsibleConverter
+from avi.migrationtools.ansible.ansible_config_converter import \
+    AviAnsibleConverterMigration
 from avi.migrationtools.avi_converter import AviConverter
+from avi.migrationtools.avi_migration_utils import get_count, \
+    PasswordPromptAction
 from avi.migrationtools.avi_orphan_object import wipe_out_not_in_use
-from avi.migrationtools.ansible.ansible_config_converter import\
-    AviAnsibleConverter
 
 LOG = logging.getLogger(__name__)
 sdk_version = getattr(avi.migrationtools, '__version__', None)
@@ -92,7 +94,12 @@ class NetscalerConverter(AviConverter):
 
         self.print_pip_and_controller_version()
         # print the arguments in input
-        LOG.info("Input parameters: %s" % ' '.join(sys.argv))
+        params = ' '.join(sys.argv)
+        if self.ns_ssh_password:
+            params = params.replace(self.ns_ssh_password, '******')
+        if self.password:
+            params = params.replace(self.password, '******')
+        LOG.info("Input parameters: %s" % params)
 
         if is_download_from_host:
             LOG.debug("Copying files from host")
@@ -112,20 +119,19 @@ class NetscalerConverter(AviConverter):
         if self.ignore_config:
             with open(self.ignore_config) as stream:
                 user_ignore = yaml.safe_load(stream)
-        # getting meta tag from superclass
-        meta = self.meta(self.tenant, self.controller_version)
         report_name = os.path.splitext(os.path.basename(source_file))[0]
         # Added dict for collecting vs for netscaler.
         vs_name_dict = dict()
         vs_name_dict['csvs'] = dict()
         vs_name_dict['lbvs'] = dict()
-        avi_config = ns_conf_converter.convert(meta, ns_config, self.tenant,
-                                               self.cloud_name, self.controller_version, output_dir,
-                                               input_dir, skipped_cmds, self.vs_state,
-                                               self.object_merge_check, report_name, self.prefix,
-                                               vs_name_dict, self.profile_path, self.redirect,
-                                               self.ns_passphrase_file, user_ignore, self.vs_level_status,
-                                               self.vrf, self.segroup)
+        avi_config = ns_conf_converter.convert(
+            ns_config, self.tenant, self.cloud_name,
+            self.controller_version, output_dir, input_dir, skipped_cmds,
+            self.vs_state, self.object_merge_check, report_name, self.prefix,
+            vs_name_dict, self.profile_path, self.redirect,
+            self.ns_passphrase_file, user_ignore, self.vs_level_status,
+            self.vrf, self.segroup)
+
         avi_config = self.process_for_utils(
             avi_config)
         # Check if flag true then skip not in use object
@@ -134,10 +140,10 @@ class NetscalerConverter(AviConverter):
         self.write_output(
             avi_config, output_dir, '%s-Output.json' % report_name)
         if self.create_ansible:
-            avi_traffic = AviAnsibleConverter(
+            avi_traffic = AviAnsibleConverterMigration(
                 avi_config, output_dir, self.prefix, self.not_in_use,
                 ns_vs_name_dict=vs_name_dict, test_vip=self.test_vip,
-                skip_types=self.ansible_skip_types)
+                skip_types=self.ansible_skip_types, controller_version=self.controller_version)
             avi_traffic.write_ansible_playbook(
                 self.ns_host_ip, self.ns_ssh_user, self.ns_ssh_password,
                 'netscaler'
@@ -175,28 +181,85 @@ if __name__ == "__main__":
             Example:
             mcqcim.key: ZcZawJ7ps0AJ+5TMDi7UA==
             avi_key.pem : foobar
-            
+
         Example to provide baseline json file absolute location:
-            netscaler_converter.py -f ns.conf --baseline_profile 
+            netscaler_converter.py -f ns.conf --baseline_profile
             /home/<'sys_conf.json' or 'ns-Output.json'>
         Usecase: Need to merge objects if there is migration of two netscaler
                  instances/box to single controller.
-        
+
         Example to patch the config after conversion:
           netscaler_converter.py -f ns.conf --patch test/patch.yaml
+          where patch.yaml file contains
+          <avi_object example Pool>:
+           - match_name: <existing name example p1>
+             patch:
+               name: <changed name example coolpool>
 
-        Example to use segroup flag 
+        Example to add the prefix to avi object name:
+          netscaler_converter.py -f ns.conf --prefix abc
+        Usecase: When two configuration is to be uploaded to same controller
+                 then in order to differentiate between the objects that will
+                 be uploaded in second time.
+
+        Example to use no object merge option:
+          netscaler_converter.py -f ns.conf --no_object_merge
+        Usecase: When we don't need to merge two same object (based on their
+                 attribute values except name)
+
+        Example to use redirect option:
+          netscaler_converter.py -f ns.conf --redirect
+        Usecase: If a http VS has no pool assigned to it but has redirect to
+                 https VS
+
+        Example to use ansible option:
+          netscaler_converter.py -f ns.conf --ansible
+        Usecase: To generate the ansible playbook for the avi configuration
+                 which can be used for upload to controller
+
+        Example to use vs level status option:
+          netscaler_converter.py -f ns.conf --vs_level_status
+        Usecase: To get the vs level status for the avi objects in excel sheet
+
+        Example to use ignore config option:
+          netscaler_converter.py -f ns.conf --ignore_config
+        Usecase: The attributes mentioned in ignore_config.yaml will appear in
+                 ignore column in excel sheet instead of skip.
+                 It will need an ignore_config.yaml file like below
+                 in the input directory defined by user
+                 <object example monitor>:
+                     <property example https>:
+                     - <attribute example 'destination'>
+        Example to use not_in_use option:
+          netscaler_converter.py -f ns.conf --not_in_use
+        Usecase: Dangling object which are not referenced by any avi object
+                 will be removed.
+
+        Example to use -s or --vs_state option:
+          netsacaler_converter.py -f ns.conf -s <enable or disable>
+        Usecase: To enable a VS after conversion to AVI.
+
+        Example to use --controller_version option:
+          netscaler_converter.py -f ns.conf --controller_version <17.2.3>
+        Usecase: To provide the version of controller.
+
+        Example to use segroup flag
+            netscalar_converter.py -f ns.conf --segroup segroup_name
+        UseCase: To add / Change segroup reference of vs
+
+        Example to use segroup flag
             netscalar_converter.py -f ns.conf --segroup segroup_name
         UseCase: To add / Change segroup reference of vs
 
         Example to use vrf flag
             netscalar_converter.py -f ns.conf --vrf vrf_name
-        UseCase: Change all the vrf reference in the configuration while conversion
+        UseCase: Change all the vrf reference in the configuration while
+        conversion
         '''
 
     parser = argparse.ArgumentParser(
         formatter_class=argparse.RawTextHelpFormatter,
-        description=(HELP_STR))
+        description=HELP_STR)
 
     # Ansible tags
     parser.add_argument('--ansible', help='Flag for create ansible file',
@@ -246,11 +309,13 @@ if __name__ == "__main__":
     parser.add_argument('--ns_passphrase_file',
                         help='Netscaler key passphrase yaml file')
     parser.add_argument('--ns_host_ip',
-                        help='host ip of Netscaler instance')                        
+                        help='host ip of Netscaler instance')
     parser.add_argument('--ns_ssh_user', help='Netscaler host ssh username')
-    parser.add_argument('--ns_ssh_password',
-                        help='Netscaler host ssh password if password based '
-                             'authentication')                             
+    parser.add_argument('--ns_ssh_password', action=PasswordPromptAction,
+                        nargs='?', help='Netscaler host ssh password if '
+                                        'password based authentication. '
+                                        'Input prompt will appear if no value '
+                                        'provided')
     parser.add_argument('-o', '--output_file_path',
                         help='Folder path for output files to be created in')
     parser.add_argument('-O', '--option',
@@ -258,9 +323,10 @@ if __name__ == "__main__":
                         help='Upload option cli-upload genarates Avi config '
                              'file auto upload will upload config to '
                              'controller', default='cli-upload')
-    parser.add_argument('-p', '--password',
-                        help='controller password for auto upload',
-                        default='avi123')
+    parser.add_argument('-p', '--password', action=PasswordPromptAction,
+                        nargs='?', help='controller password for auto upload. '
+                                        'Input prompt will appear if no value '
+                                        'provided')
     # Added command line args to execute config_patch file with related avi
     # json file location and patch location
     parser.add_argument('--patch', help='Run config_patch please provide '
@@ -271,7 +337,8 @@ if __name__ == "__main__":
     parser.add_argument('--redirect', help='redirect http vs to https vs if '
                         'there is no pool assigned', action="store_true")
     parser.add_argument('-s', '--vs_state', choices=['enable', 'disable'],
-                        help='state of VS created', default='disable')
+                        help='traffic_enabled state of VS created',
+                        default='disable')
     parser.add_argument('--segroup',
                         help='Update the available segroup ref with the'
                              'custom ref')
@@ -295,8 +362,11 @@ if __name__ == "__main__":
                         help='Update the available vrf ref with the custom vrf'
                              'reference')
     # Added command line args to execute vs_filter.py with vs_name.
-    parser.add_argument('--vs_filter', help='comma seperated names of '
-                                            'virtualservices')
+    parser.add_argument('--vs_filter',
+                        help='comma seperated names of virtualservices.\n'
+                        'Note: If patch data is supplied, vs_name should match '
+                        'the new name given in it'
+                        )
     parser.add_argument('--vs_level_status', action='store_true',
                         help='Add columns of vs reference and overall skipped '
                              'settings in status excel sheet')
@@ -308,3 +378,5 @@ if __name__ == "__main__":
         netscaler_converter.print_pip_and_controller_version()
         exit(0)
     netscaler_converter.convert()
+    print "Total Warning: ", get_count('warning')
+    print "Total Errors: ", get_count('error')
