@@ -22,7 +22,7 @@ from avi.migrationtools.netscaler_converter.ns_constants \
             OBJECT_TYPE_HTTP_POLICY_SET, STATUS_LIST, COMPLEXITY_ADVANCED,
             COMPLEXITY_BASIC, OBJECT_TYPE_APPLICATION_PERSISTENCE_PROFILE,
             OBJECT_TYPE_APPLICATION_PROFILE)
-from avi.migrationtools.avi_migration_utils import MigrationUtil
+from avi.migrationtools.avi_migration_utils import MigrationUtil, update_count
 
 LOG = logging.getLogger(__name__)
 
@@ -96,8 +96,7 @@ class NsUtil(MigrationUtil):
                 if isinstance(element_object_list, dict):
                     element_object_list = [element_object_list]
                 for element_object in element_object_list:
-                    match = [match for match in csv_writer_dict_list
-                             if
+                    match = [match for match in csv_writer_dict_list if
                              match['Line Number'] == element_object['line_no']]
                     if not match:
                         ns_complete_command = self.get_netscalar_full_command(
@@ -198,6 +197,15 @@ class NsUtil(MigrationUtil):
             avi_algorithm = 'LB_ALGORITHM_CONSISTENT_HASH_URI'
         return avi_algorithm
 
+    def update_algo_for_pools(self, algo, pg_name, avi_config):
+            pool_group = [pg for pg in avi_config['PoolGroup'] if
+                          pg['name'] == pg_name][0]
+            for member in pool_group['members']:
+                pool_name = self.get_name(member['pool_ref'])
+                pool = [pool for pool in avi_config['Pool'] if
+                        pool['name'] == pool_name][0]
+                pool['lb_algorithm'] = algo
+
     def get_avi_resp_code(self, respCode):
         """
         This function used for getting appropriate response code for avi.
@@ -227,7 +235,10 @@ class NsUtil(MigrationUtil):
                 elif code < 600:
                     avi_resp_codes.append("HTTP_5XX")
         # Get the unique dict from list.
-        return list(set(avi_resp_codes))
+        avi_resp_codes = list(set(avi_resp_codes))
+        if not avi_resp_codes:
+            avi_resp_codes = ["HTTP_ANY"]
+        return avi_resp_codes
 
     def get_conv_status(self, ns_object, skipped_list, na_list, indirect_list,
                         ignore_for_val=None, indirect_commands=None,
@@ -490,33 +501,30 @@ class NsUtil(MigrationUtil):
                             'xff_alternate_name': 'X-Forwarded-For'
                         }
                     })
-            if prop_dict.get('ssl_everywhere_enabled'):
-                if profile[0].get('http_profile'):
-                    profile[0]['http_profile'].update(
-                        {
-                            'ssl_everywhere_enabled': True,
-                            'x_forwarded_proto_enabled': True,
-                            'hsts_enabled': True,
-                            'http_to_https': True,
-                            'httponly_enabled': True,
-                            'hsts_max_age': 365,
-                            'server_side_redirect_to_https': True,
-                            'secure_cookie_enabled': True
-                        }
-                    )
-                else:
-                    profile[0].update({'http_profile':
-                        {
-                            'ssl_everywhere_enabled': True,
-                            'x_forwarded_proto_enabled': True,
-                            'hsts_enabled': True,
-                            'http_to_https': True,
-                            'httponly_enabled': True,
-                            'hsts_max_age': 365,
-                            'server_side_redirect_to_https': True,
-                            'secure_cookie_enabled': True
-                        }
-                    })
+            if profile[0].get('http_profile'):
+                profile[0]['http_profile'].update(
+                    {
+                        'x_forwarded_proto_enabled': True,
+                        'hsts_enabled': True,
+                        'http_to_https': True,
+                        'httponly_enabled': True,
+                        'hsts_max_age': 365,
+                        'server_side_redirect_to_https': True,
+                        'secure_cookie_enabled': True
+                    }
+                )
+            else:
+                profile[0].update({'http_profile':
+                    {
+                        'x_forwarded_proto_enabled': True,
+                        'hsts_enabled': True,
+                        'http_to_https': True,
+                        'httponly_enabled': True,
+                        'hsts_max_age': 365,
+                        'server_side_redirect_to_https': True,
+                        'secure_cookie_enabled': True
+                    }
+                })
 
     def object_exist(self, object_type, name, avi_config):
         '''
@@ -767,6 +775,7 @@ class NsUtil(MigrationUtil):
                     app_profile['name'])
                 return app_profile['name']
         except:
+            update_count('error')
             LOG.error("Error in convertion of httpProfile", exc_info=True)
 
     def convert_persistance_prof(self, vs, name, tenant_ref):
@@ -848,7 +857,6 @@ class NsUtil(MigrationUtil):
         policy_obj = {
             'name': vs_obj['name'] + '-redirect-policy',
             'tenant_ref': tenant_ref,
-            'enable': 'false',
             'http_request_policy': {
                 'rules': [
                     {
@@ -1634,11 +1642,15 @@ class NsUtil(MigrationUtil):
         """
         mergelist=[]
         for poolgrp in avi_config['PoolGroup']:
-            pool_member = poolgrp['members']
+            if poolgrp['name'] == 'lb-depoed1cdb.qai-pri-5984-poolgroup':
+                print 'found'
+            # do not merge the pool if it is a backup pool in the group
+            pool_member = [obj for obj in poolgrp['members'] if not
+                           obj.get('priority_label', '10') == '2']
             length = len(pool_member)
             for count in range(length):
-                pool_name = pool_member[count]['pool_ref'].split('&')[1].split(
-                    '=')[1]
+                pool_name = pool_member[count]['pool_ref'].split(
+                    '&')[1].split('=')[1]
                 if pool_name in mergelist:
                     continue
                 pool = [pl for pl in avi_config['Pool']
@@ -1647,11 +1659,10 @@ class NsUtil(MigrationUtil):
                     LOG.debug("'%s' not present" % pool_name)
                     continue
                 for count2 in range(count+1, length):
-                    pname = pool_member[count2]['pool_ref'].split('&')[1].split(
-                        '=')[1]
+                    pname = pool_member[count2]['pool_ref'].split(
+                        '&')[1].split('=')[1]
                     nextpool = [pol for pol in avi_config['Pool']
-                        if pol['name'] == pname]
-
+                                if pol['name'] == pname]
                     if not nextpool:
                         LOG.debug("'%s' not present" % pname)
                         continue
@@ -1790,7 +1801,6 @@ class NsUtil(MigrationUtil):
                 'keepalive_timeout': 30000,
                 'ssl_client_certificate_mode': "SSL_CLIENT_CERTIFICATE_NONE",
                 'http_to_https': True,
-                'spdy_enabled': False,
                 'max_bad_rps_cip_uri': 0,
                 'client_body_timeout': 30000,
                 'httponly_enabled': False,
@@ -1800,8 +1810,6 @@ class NsUtil(MigrationUtil):
                 'client_max_header_size': 12,
                 'client_max_request_size': 48,
                 'max_rps_unknown_uri': 0,
-                'ssl_everywhere_enabled': False,
-                'spdy_fwd_proxy_mode': False,
                 'post_accept_timeout': 30000,
                 'client_header_timeout': 10000,
                 'secure_cookie_enabled': False,
@@ -1816,14 +1824,11 @@ class NsUtil(MigrationUtil):
             'dos_rl_profile': {
                 'rl_profile': {
                     'client_ip_connections_rate_limit': {
-                        'count': 0,
                         'explicit_tracking': False,
-                        'period': 1,
                         'action': {
                             'status_code': "HTTP_LOCAL_RESPONSE_STATUS_CODE_429",
                             'type': "RL_ACTION_NONE"
                         },
-                        'burst_sz': 0,
                         'fine_grain': False
                     }
                 },
